/*
 * name NixieTubeClock.pde
 * date Tue Feb 26 15:34:57 JST 2013
 * 
 * Released under the 'Buy Me a Beer' license
 * (If we ever meet, you buy me a beer)
 */

import net.triring.emuduino.*;
import ddf.minim.AudioOutput;
// Pin割り当て
// constants won't change. They're used here to 
// set pin numbers:
int D2    =  2;  // Switch:Push
int S19   = 19;  // NixieTube
int S18   = 18;  // NixieTube
int S17   = 17;  // NixieTube
int S16   = 16;  // NixieTube

int year  =  0; // the year.                         
int month =  0; // the month   between 0-11          
int date  =  0; // the day of the month between 1-31.
int hrs   =  0; // the hours   between 0-23.         
int min   =  0; // the minutes between 0-59.         
int sec   =  0; // the seconds between 0-59.
int t     =  0;

void setup() {	// 一番最初に呼ばれる関数
  // ニキシー管のGUIパーツを使った日時の表示例です。
  // 年月日と時刻の取得には、javaのカレンダークラスを直接使用してます。
  initComponents();
  // initialize
  pinMode(D2, INPUT);  // Switch:Push
}

void loop() {	// 1/60秒ごとに自動実行
  java.util.Calendar calendar = java.util.Calendar.getInstance();
  year  = calendar.get(java.util.Calendar.YEAR);
  month = calendar.get(java.util.Calendar.MONTH) + 1;
  date  = calendar.get(java.util.Calendar.DATE);
  hrs   = calendar.get(java.util.Calendar.HOUR_OF_DAY);
  min   = calendar.get(java.util.Calendar.MINUTE);
  sec   = calendar.get(java.util.Calendar.SECOND);
  t = year;
  dispNixie1(0, (int)(t / 1000)); t %= 1000;
  dispNixie1(1, (int)(t / 100 )); t %= 100;
  dispNixie1(2, (int)(t / 10  )); t %= 10;
  dispNixie1(3, (int) t        );

  t = month ;
  dispNixie1(4, (int)(t / 10  )); t %= 10;
  dispNixie1(5, (int) t        );

  t = date ;
  dispNixie1(6, (int)(t / 10  )); t %= 10;
  dispNixie1(7, (int) t        );
 
  t = hrs ;
  dispNixie2(0, (int)(t / 10  )); t %= 10;
  dispNixie2(1, (int) t        );

  t = min ;
  dispNixie2(2, (int)(t / 10  )); t %= 10;
  dispNixie2(3, (int) t        );

  t = sec ;
  dispNixie2(4, (int)(t / 10  )); t %= 10;
  dispNixie2(5, (int) t        );
  delay(100);
}

void draw() {
  loop();
}

// システムの機能を完全に停止させる禁断のメソッド
// 決して、使ってはならない。
void balse() {
  System.exit(0);
}                                            

// digital INPUT
int digitalRead(int inPin) {
  int value = 0;
  switch (inPin) {
  case 2 : 
    value = switch2.isState(); 
    break;
  }
  return value;
}
// Nixie tube
void dispNixie1(int n, int value) {
  // n     : Nixie number  0 - 7
  // value : 0 - 9
  switch (n) {
  case  0 : 
    nixie1[0].setValue(value); 
    break;
  case  1 : 
    nixie1[1].setValue(value); 
    break;
  case  2 : 
    nixie1[2].setValue(value); 
    break;
  case  3 : 
    nixie1[3].setValue(value); 
    break;
  case  4 : 
    nixie1[4].setValue(value); 
    break;
  case  5 : 
    nixie1[5].setValue(value); 
    break;
  case  6 : 
    nixie1[6].setValue(value); 
    break;
  case  7 : 
    nixie1[7].setValue(value); 
    break;
  }
}
// Nixie tube
void dispNixie2(int n, int value) {
  // n     : Nixie number  0 - 5
  // value : 0 - 9
  switch (n) {
  case  0 : 
    nixie2[0].setValue(value); 
    break;
  case  1 : 
    nixie2[1].setValue(value); 
    break;
  case  2 : 
    nixie2[2].setValue(value); 
    break;
  case  3 : 
    nixie2[3].setValue(value); 
    break;
  case  4 : 
    nixie2[4].setValue(value); 
    break;
  case  5 : 
    nixie2[5].setValue(value); 
    break;
  }
}
java.awt.Image readImage(String imgfile) {
  PImage pImage = loadImage(imgfile);
  return pImage.getImage();
}

//TimerTask内部クラス
class TimerTaskAudioStop extends java.util.TimerTask {
  public void run() {
    ToneFlag = 0;
    auOut.noSound();
    auOut.mute();
  }
}

// 指定された時間だけ停止する。単位はus
void delayMicroseconds(int t_us) {
  try {
    Thread.sleep(0, t_us * 1000);  // マイクロ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// 指定された時間だけ停止する。単位はms
void delay(long t_ms) {
  try {
    Thread.sleep(t_ms);  // ミリ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// arduino起動後の時間をus単位で返す。
long micros() {
  return (SystemStartTimeNS - System.nanoTime()) * 1000;
}

// 以下は、互換性のためだけに用意したダミー関数である。
void pinMode(int pin, int mode) {
  print("pin = " + pin + ", mode = " + mode);
}

/**
 * This method is called from within the constructor to initialize the form.
 * WARNING: Do NOT modify this code. The content of this method is always
 * regenerated by the Form Editor.
 */
// @SuppressWarnings("unchecked")
// <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
private void initComponents() {
  ToneFlag = 0;

  // digital
  switch2 = new Switch(2, "Push2", 960,   0);
  nixie1 = new NixieTube[8];
  for (int r = 0; r < 1;   r++) {
    for (int c = 0; c < 8; c++) {
      int i = (r * c) + c;
      String nixie_num = "Nixie " + Integer.valueOf(i).toString();
   // nixie1[i] = new NixieTube(i, nixie_num, 168 + c * 160, 460 + r * 460);
     nixie1[i] = new NixieTube(i, nixie_num, c * 160, 460 + r * 460);
    }
  }

  nixie2 = new NixieTube[6];
  for (int r = 0; r < 1;   r++) {
    for (int c = 0; c < 6; c++) {
      int i = (r * c) + c;
      String nixie_num = "Nixie " + Integer.valueOf(i).toString();
      nixie2[i] = new NixieTube(i, nixie_num, 0 + c * 160, 0 + r * 460);
    }
  }



  minim = new ddf.minim.Minim(this);
  // get a line out from Minim, default bufferSize is 1024, default sample rate is 44100, bit depth is 16
  auOut = minim.getLineOut(ddf.minim.Minim.STEREO);
  // create a sine wave Oscillator, set to 440 Hz, at 0.5 amplitude, sample rate from line out
  // wave = new ddf.minim.signals.SineWave(440, 0.5, auOut.sampleRate());
  wave = new ddf.minim.signals.SquareWave(440, 0.5, auOut.sampleRate());
  // set the portamento speed on the oscillator to 200 milliseconds
  wave.portamento(200);

  DelayTimer = new java.util.Timer(); //タイマーインスタンス作成

  // add the oscillator to the line out
  auOut.addSignal(wave);
  auOut.noSound();  // Disables all signals currently attached to this.
  auOut.mute();
  ToneFlag = 0;

  SystemStartTimeMS = System.currentTimeMillis();  //ミリ秒で表される現在の時間を返します。
  SystemStartTimeNS = System.nanoTime(); // 利用可能でもっとも正確なシステムタイマーの現在の値をナノ秒単位で返します。
  size(320, 128);
  setLocation(0, 600);
  background(15, 115, 145);
  fill(15, 115, 145);  
  rect(0, 0, 320, 128);
  fill( 30, 160, 145);  
  textSize(60); 
  text(TITLE, 24, 68);
  fill( 45, 192, 145);  
  textSize(60); 
  text(TITLE, 22, 66);
  fill(255, 255, 255);  
  textSize(60); 
  text(TITLE, 20, 64);
  fill(255, 255, 255);  
  textSize(18); 
  text("Ver. " + Arduino.getVersion(), 120, 108);
  setLocation(0, 600);
}// </editor-fold>//GEN-END:initComponents

// 変数の宣言部
// Variables declaration - do not modify//GEN-BEGIN:variables

private String TITLE   = "Emuduino";
private String VERSION = "0.49"; // Arduino.javaで使用するプロパティファイル中のバージョンを参照する。

private long SystemStartTimeMS;
private long SystemStartTimeNS;

private static final int HIGH   = 0x1;
private static final int LOW    = 0x0;
private static final int INPUT  = 0x0;
private static final int OUTPUT = 0x1;

// private static final int A0     = 0x0;
// private static final int A1     = 0x1;
// private static final int A2     = 0x2;
// private static final int A3     = 0x3;
// private static final int A4     = 0x4;
// private static final int A5     = 0x5;
// private static final int A6     = 0x6;
// private static final int A7     = 0x7;
// private static final int A8     = 0x8;
// private static final int A9     = 0x9;

private int ToneFlag = 0;  // 0の場合は停止状態。再生中は，その周波数を保持

private java.util.Timer            DelayTimer;
private java.util.TimerTask        TimerTaskAudioStop;
private ddf.minim.Minim            minim; // http://code.compartmental.net/minim/javadoc/
private ddf.minim.AudioOutput      auOut;
// private ddf.minim.signals.SawWave      wave; // A saw wave is characterized by rising edge followed by a sharp drop off.
// private ddf.minim.signals.SineWave     wave; // A sine wave is a smooth curve that oscillates between -1 and 1.
private ddf.minim.signals.SquareWave   wave; // A square wave alternates between 1 and -1 at regular intervals.
// private ddf.minim.signals.TriangleWave wave; // A triangle wave oscillates between 1 and -1 following a straight line.

private Switch switch2;
private NixieTube[] nixie1;
private NixieTube[] nixie2;


// End of variables declaration//GEN-END:variables

