/*
 * name LedCubePanel.pde
 * date Tue Feb 05 16:48:00 JST 2013
 * 
 * Released under the 'Buy Me a Beer' license
 * (If we ever meet, you buy me a beer)
 */

import net.triring.emuduino.*;
import ddf.minim.AudioOutput;
// Pin割り当て
// constants won't change. They're used here to 
// set pin numbers:

int Count = 0;
int Color = 0;

final static int  HEIGHT = 9; //高さ(z軸)
final static int  DEPTH  = 9; //奥行(y軸)
final static int  WIDTH  = 9; //幅(x軸)


final static int  ALL = HEIGHT * DEPTH * WIDTH;

final static int half_width = int(0.5 * WIDTH);
final static int half_depth = int(0.5 * DEPTH);

void setup() {	// 一番最初に呼ばれる関数
  // 9x9x9 の Led Cube で複雑な波の動きを表示します。
  initComponents();
  // initialize
}

void loop() {	// 1/60秒ごとに自動実行
/**
 * 波
 */
  double xfloat;
  double yfloat;
  double zfloat;
  int zint;

  Count = (Count ) % (HEIGHT );

  for(int y = 0 ; y < DEPTH ; y++) {
    yfloat = (y - half_depth);
    for(int x = 0 ; x < WIDTH ; x++) {
      xfloat = (x - half_width);
      zfloat = (Math.cos(xfloat * xfloat + yfloat * yfloat - Count * 10) + 1) * .5 * HEIGHT;
      zint = constrain((int)(zfloat + 0.5), 0, HEIGHT - 1);
      for (int z = 0 ; z < HEIGHT; z++) {
        if (z == zint) {
          ledcube1.setColor(x, y, z,0xffff0000);
        } else {
          ledcube1.setColor(x, y, z,0x00000000);
        }
      }
    }
  }
  Count++;
  delay(500);               // wait for a second
}

void draw() {
  loop();
}

// システムの機能を完全に停止させる禁断のバルスメソッド
// 決して、使ってはならない。
void Balse() {
  System.exit(0);
}                                            

// digital OUTPUT
void digitalWrite(int outPin, int value) {
}

java.awt.Image readImage(String imgfile) {
  PImage pImage = loadImage(imgfile);
  return pImage.getImage();
}

//TimerTask内部クラス
class TimerTaskAudioStop extends java.util.TimerTask {
  public void run() {
    ToneFlag = 0;
    auOut.noSound();
    auOut.mute();
  }
}

// 指定された時間だけ停止する。単位はus
void delayMicroseconds(int t_us) {
  try {
    Thread.sleep(0, t_us * 1000);  // マイクロ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// 指定された時間だけ停止する。単位はms
void delay(long t_ms) {
  try {
    Thread.sleep(t_ms);  // ミリ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// arduino起動後の時間をus単位で返す。
long micros() {
  return (SystemStartTimeNS - System.nanoTime()) * 1000;
}

// 以下は、互換性のためだけに用意したダミー関数である。
void pinMode(int pin, int mode) {
   print("pin = " + pin + ", mode = " + mode);
}

/**
 * This method is called from within the constructor to initialize the form.
 * WARNING: Do NOT modify this code. The content of this method is always
 * regenerated by the Form Editor.
 */
// @SuppressWarnings("unchecked")
// <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
private void initComponents() {
  ToneFlag = 0;

  // digital
  ledcube1 = new LedCube(1, "LedCube Wave", 9, 9, 9, 0, 0);

  minim = new ddf.minim.Minim(this);
  // get a line out from Minim, default bufferSize is 1024, default sample rate is 44100, bit depth is 16
  auOut = minim.getLineOut(ddf.minim.Minim.STEREO);
  // create a sine wave Oscillator, set to 440 Hz, at 0.5 amplitude, sample rate from line out
  // wave = new ddf.minim.signals.SineWave(440, 0.5, auOut.sampleRate());
  wave = new ddf.minim.signals.SquareWave(440, 0.5, auOut.sampleRate());
  // set the portamento speed on the oscillator to 200 milliseconds
  wave.portamento(200);

  DelayTimer = new java.util.Timer(); //タイマーインスタンス作成

  // add the oscillator to the line out
  auOut.addSignal(wave);
  auOut.noSound();  // Disables all signals currently attached to this.
  auOut.mute();
  ToneFlag = 0;

  SystemStartTimeMS = System.currentTimeMillis();  //ミリ秒で表される現在の時間を返します。
  SystemStartTimeNS = System.nanoTime(); // 利用可能でもっとも正確なシステムタイマーの現在の値をナノ秒単位で返します。
  size(320, 128);
  setLocation(0, 600);
  background(15, 115, 145);
  fill( 15, 115, 145);  rect(0, 0, 320, 128);
  fill( 30, 160, 145);  textSize(60); text(TITLE , 24, 68);
  fill( 45, 192, 145);  textSize(60); text(TITLE , 22, 66);
  fill(255, 255, 255);  textSize(60); text(TITLE , 20, 64);
  fill(255, 255, 255);  textSize(18); text("Ver. " + Arduino.getVersion(), 120, 108);
  setLocation(0, 600);
}// </editor-fold>//GEN-END:initComponents

// 変数の宣言部
// Variables declaration - do not modify//GEN-BEGIN:variables

private String TITLE   = "Emuduino";
private String VERSION = "0.49"; // Arduino.javaで使用するプロパティファイル中のバージョンを参照する。

private long SystemStartTimeMS;
private long SystemStartTimeNS;

private static final int HIGH   = 0x1;
private static final int LOW    = 0x0;
private static final int INPUT  = 0x0;
private static final int OUTPUT = 0x1;

private int ToneFlag = 0;  // 0の場合は停止状態。再生中は，その周波数を保持

private java.util.Timer            DelayTimer;
private java.util.TimerTask        TimerTaskAudioStop;
private ddf.minim.Minim            minim; // http://code.compartmental.net/minim/javadoc/
private ddf.minim.AudioOutput      auOut;
// private ddf.minim.signals.SawWave      wave; // A saw wave is characterized by rising edge followed by a sharp drop off.
// private ddf.minim.signals.SineWave     wave; // A sine wave is a smooth curve that oscillates between -1 and 1.
private ddf.minim.signals.SquareWave   wave; // A square wave alternates between 1 and -1 at regular intervals.
// private ddf.minim.signals.TriangleWave wave; // A triangle wave oscillates between 1 and -1 following a straight line.

private LedCube ledcube1;

// End of variables declaration//GEN-END:variables
