/*
 * name SorobanClock.pde
 * date Sat Nov 30 21:37:20 JST 2013
 * 
 * Released under the 'Buy Me a Beer' license
 * (If we ever meet, you buy me a beer)
 */

import net.triring.emuduino.*;
import ddf.minim.AudioOutput;
// Pin割り当て
// constants won't change. They're used here to 
// set pin numbers:
int   S0 =   0;  // Serial
int   S1 =   1;  // Serial
int  S19 =  19;  // LedMatrix
int  S18 =  18;  // LedMatrix

// 2038年1月19日3時14分7秒
int year  = 2038; // the year.                         
int month =    1; // the month   between 0-11          
int date  =   19; // the day of the month between 1-31.
int hrs   =    3; // the hours   between 0-23.         
int min   =   14; // the minutes between 0-59.         
int sec   =    7; // the seconds between 0-59.
int t     =  0;

private int Matrix[][] = {
  { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 },
  { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 },
  { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 },
  { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 },
  { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 },
  { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 },
  { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 },
  { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 }
};

void setup() {	// 一番最初に呼ばれる関数
  // LedMatrixのGUIパーツを使って年月日時分秒を表示します。
  // 年月日と時刻の取得には、javaのカレンダークラスを直接使用してます。
  // 表示は、そろばんと同じで、Ledの縦一列が、一桁の数字を表します。
  // 2038年1月19日3時14分7秒 は、以下のように表示されます。
  //   2 0 3 8 0 1 1 9   3   1 4   0 7
  // [  , , , , , , , , , , , , , , ,  ]
  // [  , , ,O, , , ,O, , , , , , , ,O ]
  // [  , , , , , , , , , , , , , , ,  ]
  // [ O, ,O,O, ,O,O,O, ,O, ,O,O, , ,O ]
  // [ O, ,O,O, , , ,O, ,O, , ,O, , ,O ]
  // [  , ,O,O, , , ,O, ,O, , ,O, , ,  ]
  // [  , , , , , , ,O, , , , ,O, , ,  ]
  // [  , , , , , , , , , , , , , , ,  ]
  initComponents();

}

void loop() {	// 1/60秒ごとに自動実行
  setTime();
  dispTime(Matrix, net.triring.emuduino.LedMatrix.RED);
  delay(1000);
}

void setTime() {
  java.util.Calendar calendar = java.util.Calendar.getInstance();

  year  = calendar.get(java.util.Calendar.YEAR);
  month = calendar.get(java.util.Calendar.MONTH) + 1;
  date  = calendar.get(java.util.Calendar.DATE);
  hrs   = calendar.get(java.util.Calendar.HOUR_OF_DAY);
  min   = calendar.get(java.util.Calendar.MINUTE);
  sec   = calendar.get(java.util.Calendar.SECOND);

  String s = String.format("%04d/%02d/%02d %02d:%02d:%02d", 
                     year ,month, date, hrs, min, sec);
 Serial.println(s);
 ledmatrix1.setTitle(s);
  t = year;

  setDigit(0, (int)(t / 1000)); t %= 1000;
  setDigit(1, (int)(t / 100 )); t %= 100;
  setDigit(2, (int)(t / 10  )); t %= 10;
  setDigit(3, (int) t        );

  t = month ;
  setDigit(4, (int)(t / 10  )); t %= 10;
  setDigit(5, (int) t        );

  t = date ;
  setDigit(6, (int)(t / 10  )); t %= 10;
  setDigit(7, (int) t        );
 
  t = hrs ;
  setDigit(8, (int)(t / 10  )); t %= 10;
  setDigit(9, (int) t        );

  t = min ;
  setDigit(11, (int)(t / 10  )); t %= 10;
  setDigit(12, (int) t        );

  t = sec ;
  setDigit(14, (int)(t / 10  )); t %= 10;
  setDigit(15, (int) t        );  
}

void setDigit(int  d, int n) {
  for (int i = 0; i < 8; i++) {
    Matrix[i][d] = 0;
  }
  // 5 以上の場合
  if (n > 5) Matrix[1][ d] = 1;
  switch (n % 5) {
    case  4 : Matrix[6][ d] = 1;
    case  3 : Matrix[5][ d] = 1;
    case  2 : Matrix[4][ d] = 1;
    case  1 : Matrix[3][ d] = 1; 
  }
}

void dispTime(int data[][], int c) {
  for (int j = 0; j < 16; j++) {
    for (int i = 0; i < 8 ; i++) {
      if (data[i][j] != 0) {
         ledmatrix1.setPixel(j, i, c);
      } else {
         ledmatrix1.setPixel(j, i, net.triring.emuduino.LedMatrix.OFF);
      }
    }           
  }  
}

void draw() {
  loop();
}

// システムの機能を完全に停止させる禁断のメソッド
// 決して、使ってはならない。
void balse() {
  System.exit(0);
}                                            

java.awt.Image readImage(String imgfile) {
  PImage pImage = loadImage(imgfile);
  return pImage.getImage();
}

//TimerTask内部クラス
class TimerTaskAudioStop extends java.util.TimerTask {
  public void run() {
    ToneFlag = 0;
    auOut.noSound();
    auOut.mute();
  }
}

// 指定された時間だけ停止する。単位はus
void delayMicroseconds(int t_us) {
  try {
    Thread.sleep(0, t_us * 1000);  // マイクロ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// 指定された時間だけ停止する。単位はms
void delay(long t_ms) {
  try {
    Thread.sleep(t_ms);  // ミリ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// arduino起動後の時間をus単位で返す。
long micros() {
  return (SystemStartTimeNS - System.nanoTime()) * 1000;
}

// 以下は、互換性のためだけに用意したダミー関数である。
void pinMode(int pin, int mode) {
   print("pin = " + pin + ", mode = " + mode);
}

/**
 * This method is called from within the constructor to initialize the form.
 * WARNING: Do NOT modify this code. The content of this method is always
 * regenerated by the Form Editor.
 */
// @SuppressWarnings("unchecked")
// <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
private void initComponents() {
  ToneFlag = 0;

  // digital
  Serial = new SerialConsole(420, 0);
  ledmatrix1 = new LedMatrix(1, "LedMatrix Clock", 16, 8, 0, 0);
  

  minim = new ddf.minim.Minim(this);
  // get a line out from Minim, default bufferSize is 1024, default sample rate is 44100, bit depth is 16
  auOut = minim.getLineOut(ddf.minim.Minim.STEREO);
  // create a sine wave Oscillator, set to 440 Hz, at 0.5 amplitude, sample rate from line out
  // wave = new ddf.minim.signals.SineWave(440, 0.5, auOut.sampleRate());
  wave = new ddf.minim.signals.SquareWave(440, 0.5, auOut.sampleRate());
  // set the portamento speed on the oscillator to 200 milliseconds
  wave.portamento(200);

  DelayTimer = new java.util.Timer(); //タイマーインスタンス作成

  // add the oscillator to the line out
  auOut.addSignal(wave);
  auOut.noSound();  // Disables all signals currently attached to this.
  auOut.mute();
  ToneFlag = 0;

  SystemStartTimeMS = System.currentTimeMillis();  //ミリ秒で表される現在の時間を返します。
  SystemStartTimeNS = System.nanoTime(); // 利用可能でもっとも正確なシステムタイマーの現在の値をナノ秒単位で返します。
  size(320, 128);
  setLocation(0, 600);
  background(15, 115, 145);
  fill( 15, 115, 145);  rect(0, 0, 320, 128);
  fill( 30, 160, 145);  textSize(60); text(TITLE , 24, 68);
  fill( 45, 192, 145);  textSize(60); text(TITLE , 22, 66);
  fill(255, 255, 255);  textSize(60); text(TITLE , 20, 64);
  fill(255, 255, 255);  textSize(18); text("Ver. " + Arduino.getVersion(), 120, 108);
  setLocation(0, 600);
}// </editor-fold>//GEN-END:initComponents

// 変数の宣言部
// Variables declaration - do not modify//GEN-BEGIN:variables

private String TITLE   = "Emuduino";
private String VERSION = "0.49"; // Arduino.javaで使用するプロパティファイル中のバージョンを参照する。

private long SystemStartTimeMS;
private long SystemStartTimeNS;

private static final int HIGH   = 0x1;
private static final int LOW    = 0x0;
private static final int INPUT  = 0x0;
private static final int OUTPUT = 0x1;

private int ToneFlag = 0;  // 0の場合は停止状態。再生中は，その周波数を保持

private java.util.Timer            DelayTimer;
private java.util.TimerTask        TimerTaskAudioStop;
private ddf.minim.Minim            minim; // http://code.compartmental.net/minim/javadoc/
private ddf.minim.AudioOutput      auOut;
// private ddf.minim.signals.SawWave      wave; // A saw wave is characterized by rising edge followed by a sharp drop off.
// private ddf.minim.signals.SineWave     wave; // A sine wave is a smooth curve that oscillates between -1 and 1.
private ddf.minim.signals.SquareWave   wave; // A square wave alternates between 1 and -1 at regular intervals.
// private ddf.minim.signals.TriangleWave wave; // A triangle wave oscillates between 1 and -1 following a straight line.

private SerialConsole Serial;
private LedMatrix ledmatrix1;


// End of variables declaration//GEN-END:variables


