/*
 * name random_fade.pde
 * date Fri Aug 02 22:52:31 JST 2013
 * 
 * Released under the 'Buy Me a Beer' license
 * (If we ever meet, you buy me a beer)
 */

/*
  Random fade 
  for ikkeiplexing shield
  and 8x16duino Tichaino
  Copyright ikkei 2012.2.29
*/

import net.triring.emuduino.*;
import ddf.minim.AudioOutput;
// Pin割り当て
// constants won't change. They're used here to 
// set pin numbers:
int   D2 =   2;  // Switch:Push
int   D3 =   3;  // Switch:Push
int  SPK =  10;  // Speaker:
int  S19 =  19;  // LedMatrix
int  S18 =  18;  // LedMatrix

final static int X_MAX = 15;
final static int Y_MAX =  7;

final static int THMAX   = 20;
final static int SEEDMAX = 10;
final static int WAIT    = 20;
int th=0;
int dt=1;
int r_state = 0;
int cnt = WAIT;
int seed = 0;

char[] heart = {
  0x0630,  // B0000110001100000
  0x0F78,  // B0001111011110000
  0x0FF8,  // B0001111111110000
  0x0FF8,  // B0001111111110000
  0x07F0,  // B0000111111100000
  0x03E0,  // B0000011111000000
  0x01C0,  // B0000001110000000
  0x0080   // B0000000100000000
};


void change_seed(){
  seed++;
  if (seed > SEEDMAX){
    seed = 0;
  }
}

void change_state(){
  switch (r_state){
    case 0: cnt--;
            if (cnt == 0){
              r_state = 1;
              change_seed();
            }
            break;
    case 1: th++;
            if (th > THMAX){
              r_state = 2;
              cnt = WAIT;
            }
            break;
    case 2: cnt--;
            if (cnt == 0){
              r_state = 3;
              change_seed();
            }
            break;
    case 3: th--;
            if (th == 0){
              r_state = 0;
              cnt = WAIT;
            }
            break;
  }
}

void setup() {	// 一番最初に呼ばれる関数
  // "ちっちゃい～の" のサンプルを引用しました。
  // 8×8ドット マトリクスLEDを2個使用した
  // 20 x 40 mmの表示モジュールのキット
  //  http://ikkei.akiba.coocan.jp/ikkei_Electronics/MTM07.html
  //  random_fade.ino
  
  initComponents();
  // initialize
  pinMode(D2,   INPUT);  // Switch:Push
  pinMode(D3,   INPUT);  // Switch:Push
  pinMode(SPK, OUTPUT);  // Speaker:
}

void loop() {	// 1/60秒ごとに自動実行
  change_state(); 
  randomSeed(seed); // seed == 0 : invalid
  for (byte y=0; y<8; y++){
    for (byte x=0; x<16; x++){
      if ((0 != (bitRead(heart[y], 15-x)) ^ (random(0,20) < th))) {
        ledmatrix1.setPixel(x, y, net.triring.emuduino.LedMatrix.GREEN );
      } else {
        ledmatrix1.setPixel(x, y, net.triring.emuduino.LedMatrix.OFF   );
      }
    }
  }
  delay(10);
}

int bitRead(char ch, int x) {
   return (0x0001 & (ch >> x));
}

/* -----------------------------------------------------------------  */
void Vwrite(int x, char data) {
    for(int y=0; y<8; y++){
      if(0 != (data & (1<<y))){
        ledmatrix1.setPixel(x, y, net.triring.emuduino.LedMatrix.GREEN);
      } else {
        ledmatrix1.setPixel(x, y, net.triring.emuduino.LedMatrix.OFF  );
      }
    }
}

/* -----------------------------------------------------------------  */
/** Shift left the screen completely
 */
void ShiftLeft() {
    for(int x=1;x<16;x++) {  
        for(int y=0;y<8;y++) { 
            ledmatrix1.setPixel(x-1,y,ledmatrix1.getPixel(x, y));
        }
    }
}


/* -----------------------------------------------------------------  */
/** Shift right the screen completely
 */
void ShiftRight() {
    for(int x=1;x<16;x++) {  
        for(int y=0;y<8;y++) { 
            ledmatrix1.setPixel(16-x, y, ledmatrix1.getPixel(15-x,y));
        }
    }
}


void draw() {
  loop();
}

// システムの機能を完全に停止させる禁断のメソッド
// 決して、使ってはならない。
void balse() {
  System.exit(0);
}                                            

// digital INPUT
int digitalRead(int inPin) {
    int value = 0;
    switch (inPin) {
        case 2 : value = switch2.isState(); break;
        case 3 : value = switch3.isState(); break;
    }
    return value;
}
// Speakerで使用する関数群

void tone(int pin, int frequency) {
  if (ToneFlag == frequency) return;
  ToneFlag = frequency;
  speaker10.setFrequency(frequency);
  wave.setFreq((float)frequency);
  auOut.unmute();
  auOut.sound();
}

void tone(int pin, int frequency, long duration) {
  if (ToneFlag == frequency) return;
  ToneFlag = frequency;
  wave.setFreq((float)frequency);
  auOut.unmute();
  auOut.sound();
  TimerTaskAudioStop timerTask = new TimerTaskAudioStop(); //タイマータスクインスタンス作成
  //タイマー開始
  DelayTimer.schedule(timerTask, duration);
}

void noTone(int pin) {
  ToneFlag = 0;
  speaker10.setFrequency(0);
  auOut.mute();
  auOut.noSound();
}
java.awt.Image readImage(String imgfile) {
  PImage pImage = loadImage(imgfile);
  return pImage.getImage();
}

//TimerTask内部クラス
class TimerTaskAudioStop extends java.util.TimerTask {
  public void run() {
    ToneFlag = 0;
    auOut.noSound();
    auOut.mute();
  }
}

// 指定された時間だけ停止する。単位はus
void delayMicroseconds(int t_us) {
  try {
    Thread.sleep(0, t_us * 1000);  // マイクロ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// 指定された時間だけ停止する。単位はms
void delay(long t_ms) {
  try {
    Thread.sleep(t_ms);  // ミリ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// arduino起動後の時間をus単位で返す。
long micros() {
  return (SystemStartTimeNS - System.nanoTime()) * 1000;
}

// 以下は、互換性のためだけに用意したダミー関数である。
void pinMode(int pin, int mode) {
   print("pin = " + pin + ", mode = " + mode);
}

/**
 * This method is called from within the constructor to initialize the form.
 * WARNING: Do NOT modify this code. The content of this method is always
 * regenerated by the Form Editor.
 */
// @SuppressWarnings("unchecked")
// <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
private void initComponents() {
  ToneFlag = 0;

  // digital
                   switch2 = new Switch(2 , "Push2",    0,  240);
                   switch3 = new Switch(3 , "Push3",  240,  240);
                 speaker10 = new Speaker(10 , "speaker10",  384,  240);
  ledmatrix1 = new LedMatrix(1, "LedMatrix 1", 16, 8, 0, 0);
  

  minim = new ddf.minim.Minim(this);
  // get a line out from Minim, default bufferSize is 1024, default sample rate is 44100, bit depth is 16
  auOut = minim.getLineOut(ddf.minim.Minim.STEREO);
  // create a sine wave Oscillator, set to 440 Hz, at 0.5 amplitude, sample rate from line out
  // wave = new ddf.minim.signals.SineWave(440, 0.5, auOut.sampleRate());
  wave = new ddf.minim.signals.SquareWave(440, 0.5, auOut.sampleRate());
  // set the portamento speed on the oscillator to 200 milliseconds
  wave.portamento(200);

  DelayTimer = new java.util.Timer(); //タイマーインスタンス作成

  // add the oscillator to the line out
  auOut.addSignal(wave);
  auOut.noSound();  // Disables all signals currently attached to this.
  auOut.mute();
  ToneFlag = 0;

  SystemStartTimeMS = System.currentTimeMillis();  //ミリ秒で表される現在の時間を返します。
  SystemStartTimeNS = System.nanoTime(); // 利用可能でもっとも正確なシステムタイマーの現在の値をナノ秒単位で返します。
  size(320, 128);
  setLocation(0, 600);
  background(15, 115, 145);
  fill( 15, 115, 145);  rect(0, 0, 320, 128);
  fill( 30, 160, 145);  textSize(60); text(TITLE , 24, 68);
  fill( 45, 192, 145);  textSize(60); text(TITLE , 22, 66);
  fill(255, 255, 255);  textSize(60); text(TITLE , 20, 64);
  fill(255, 255, 255);  textSize(18); text("Ver. " + Arduino.getVersion(), 120, 108);
  setLocation(0, 600);
}// </editor-fold>//GEN-END:initComponents

// 変数の宣言部
// Variables declaration - do not modify//GEN-BEGIN:variables

private String TITLE   = "Emuduino";
private String VERSION = "0.49"; // Arduino.javaで使用するプロパティファイル中のバージョンを参照する。

private long SystemStartTimeMS;
private long SystemStartTimeNS;

private static final int HIGH   = 0x1;
private static final int LOW    = 0x0;
private static final int INPUT  = 0x0;
private static final int OUTPUT = 0x1;

private int ToneFlag = 0;  // 0の場合は停止状態。再生中は，その周波数を保持

private java.util.Timer            DelayTimer;
private java.util.TimerTask        TimerTaskAudioStop;
private ddf.minim.Minim            minim; // http://code.compartmental.net/minim/javadoc/
private ddf.minim.AudioOutput      auOut;
// private ddf.minim.signals.SawWave      wave; // A saw wave is characterized by rising edge followed by a sharp drop off.
// private ddf.minim.signals.SineWave     wave; // A sine wave is a smooth curve that oscillates between -1 and 1.
private ddf.minim.signals.SquareWave   wave; // A square wave alternates between 1 and -1 at regular intervals.
// private ddf.minim.signals.TriangleWave wave; // A triangle wave oscillates between 1 and -1 following a straight line.

private Switch switch2;
private Switch switch3;
private Speaker speaker10;
private LedMatrix ledmatrix1;

// End of variables declaration//GEN-END:variables



