/*
 * name Tutorial02.pde
 * date Sat Mar 30 23:54:55 JST 2013
 * 
 * Released under the 'Buy Me a Beer' license
 * (If we ever meet, you buy me a beer)
 */

import net.triring.emuduino.*;
import ddf.minim.AudioOutput;
// Pin割り当て
// constants won't change. They're used here to 
// set pin numbers:
int    SW2 =   2;  // Switch:Push
int  SPK10 =  10;  // Speaker:
int  LED13 =  13;  // Arduino:diecimila
int     S0 =   0;  // Serial
int     S1 =   1;  // Serial

void setup() {	// 一番最初に呼ばれる関数
// Tutorial02
// ボタンが押されると，LEDが点灯します。
// 実行中の状態をシリアルコンソールに表示します。
// 課題:ボタンを押すと，音を再生するようにせよ。

  initComponents();   // GUIパーツを初期化します。
  // initialize
  // 使用する入出力ポートの設定を行います。
  pinMode(SW2,   INPUT );  // Switch:Push
  pinMode(SPK10, OUTPUT);  // Speaker:
  pinMode(LED13, OUTPUT);  // Arduino:diecimila
  Serial.begin(9600);      // シリアルコンソールとの通信速度を設定します。。
}

void loop() {	// 1/60秒ごとに自動実行
  int state;         // ボタンの状態を保存します。
  int freq_hz = 440; // 再生周波数

  state = digitalRead(SW2); // ボタンの状態を読み取ります。
  Serial.print("\nswitch = ");
  Serial.print(state);

  if (HIGH == state) {          // ボタンが押されたら，以下を実行します。
    digitalWrite(LED13, HIGH);  // LEDを点灯します。
    Serial.print(", freq = ");
    Serial.print(freq_hz);
  tone(SPK10, freq_hz);         // freq_hzで指定した周波数で音を再生します。 
    delay(500);                 // 500ms間の時間待ち
    noTone(SPK10);              // 音の再生を停止します。
   } else {
    digitalWrite(LED13, LOW );  // LEDを消灯します。
    delay(500);                 // 500ms間の時間待ち  
   }
}


void draw() {
  loop();
}

// システムの機能を完全に停止させる禁断のメソッド
// 決して、使ってはならない。
void balse() {
  System.exit(0);
}                                            

// digital INPUT
int digitalRead(int inPin) {
    int value = 0;
    switch (inPin) {
        case 2 : value = switch2.isState(); break;
    }
    return value;
}
// digital OUTPUT
void digitalWrite(int outPin, int value) {
    switch (outPin) {
        case 13 : arduino13.setState(value); break;
    }
}
// analog INPUT
int analogRead(int inPin) {
    int value = 0;
    switch (inPin) {
        case   0 : value = slider0.getValue(); break;
    }
    return value;
}
// Speakerで使用する関数群

void tone(int pin, int frequency) {
  if (ToneFlag == frequency) return;
  ToneFlag = frequency;
  speaker10.setFrequency(frequency);
  wave.setFreq((float)frequency);
  auOut.unmute();
  auOut.sound();
}

void tone(int pin, int frequency, long duration) {
  if (ToneFlag == frequency) return;
  ToneFlag = frequency;
  wave.setFreq((float)frequency);
  auOut.unmute();
  auOut.sound();
  TimerTaskAudioStop timerTask = new TimerTaskAudioStop(); //タイマータスクインスタンス作成
  //タイマー開始
  DelayTimer.schedule(timerTask, duration);
}

void noTone(int pin) {
  ToneFlag = 0;
  speaker10.setFrequency(0);
  auOut.mute();
  auOut.noSound();
}
java.awt.Image readImage(String imgfile) {
  PImage pImage = loadImage(imgfile);
  return pImage.getImage();
}

//TimerTask内部クラス
class TimerTaskAudioStop extends java.util.TimerTask {
  public void run() {
    ToneFlag = 0;
    auOut.noSound();
    auOut.mute();
  }
}

// 指定された時間だけ停止する。単位はus
void delayMicroseconds(int t_us) {
  try {
    Thread.sleep(0, t_us * 1000);  // マイクロ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// 指定された時間だけ停止する。単位はms
void delay(long t_ms) {
  try {
    Thread.sleep(t_ms);  // ミリ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// arduino起動後の時間をus単位で返す。
long micros() {
  return (SystemStartTimeNS - System.nanoTime()) * 1000;
}

// 以下は、互換性のためだけに用意したダミー関数である。
void pinMode(int pin, int mode) {
   print("pin = " + pin + ", mode = " + mode);
}

/**
 * This method is called from within the constructor to initialize the form.
 * WARNING: Do NOT modify this code. The content of this method is always
 * regenerated by the Form Editor.
 */
// @SuppressWarnings("unchecked")
// <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
private void initComponents() {
  ToneFlag = 0;

  // digital
    switch2 = new Switch(2   , "Push2",     300,    0);
  speaker10 = new Speaker(10 , "speaker10", 420,    0);
  arduino13 = new Arduino(13 , "arduino13",   0,    0, "controller_arduino_uno_r3");
//  slider0 = new Slider(0   , "INPUT0",      0,  256, javax.swing.JSlider.HORIZONTAL);
//  slider0.setSize(384, 120);
     Serial = new SerialConsole(            600,    0);
  

  minim = new ddf.minim.Minim(this);
  // get a line out from Minim, default bufferSize is 1024, default sample rate is 44100, bit depth is 16
  auOut = minim.getLineOut(ddf.minim.Minim.STEREO);
  // create a sine wave Oscillator, set to 440 Hz, at 0.5 amplitude, sample rate from line out
  // wave = new ddf.minim.signals.SineWave(440, 0.5, auOut.sampleRate());
  wave = new ddf.minim.signals.SquareWave(440, 0.5, auOut.sampleRate());
  // set the portamento speed on the oscillator to 200 milliseconds
  wave.portamento(200);

  DelayTimer = new java.util.Timer(); //タイマーインスタンス作成

  // add the oscillator to the line out
  auOut.addSignal(wave);
  auOut.noSound();  // Disables all signals currently attached to this.
  auOut.mute();
  ToneFlag = 0;

  SystemStartTimeMS = System.currentTimeMillis();  //ミリ秒で表される現在の時間を返します。
  SystemStartTimeNS = System.nanoTime(); // 利用可能でもっとも正確なシステムタイマーの現在の値をナノ秒単位で返します。
  size(320, 128);
  setLocation(0, 600);
  background(15, 115, 145);
  fill(15, 115, 145);  rect(0, 0, 320, 128);
  fill( 30, 160, 145);  textSize(60); text(TITLE , 24, 68);
  fill( 45, 192, 145);  textSize(60); text(TITLE , 22, 66);
  fill(255, 255, 255);  textSize(60); text(TITLE , 20, 64);
  fill(255, 255, 255);  textSize(18); text("Ver. " + Arduino.getVersion(), 120, 108);
  setLocation(0, 600);
}// </editor-fold>//GEN-END:initComponents

// 変数の宣言部
// Variables declaration - do not modify//GEN-BEGIN:variables

private String TITLE   = "Emuduino";
private String VERSION = "0.49"; // Arduino.javaで使用するプロパティファイル中のバージョンを参照する。

private long SystemStartTimeMS;
private long SystemStartTimeNS;

private static final int HIGH   = 0x1;
private static final int LOW    = 0x0;
private static final int INPUT  = 0x0;
private static final int OUTPUT = 0x1;

private int ToneFlag = 0;  // 0の場合は停止状態。再生中は，その周波数を保持

private java.util.Timer            DelayTimer;
private java.util.TimerTask        TimerTaskAudioStop;
private ddf.minim.Minim            minim; // http://code.compartmental.net/minim/javadoc/
private ddf.minim.AudioOutput      auOut;
// private ddf.minim.signals.SawWave      wave; // A saw wave is characterized by rising edge followed by a sharp drop off.
// private ddf.minim.signals.SineWave     wave; // A sine wave is a smooth curve that oscillates between -1 and 1.
private ddf.minim.signals.SquareWave   wave; // A square wave alternates between 1 and -1 at regular intervals.
// private ddf.minim.signals.TriangleWave wave; // A triangle wave oscillates between 1 and -1 following a straight line.

private Switch switch2;
private Speaker speaker10;
private Arduino arduino13;
private Slider slider0;
private SerialConsole Serial;


// End of variables declaration//GEN-END:variables


